#!/usr/bin/perl

###############################################################################
#
# Copyright (C) 2013 ClearOne Communications, Inc. All rights reserved.
#
#     Confidential -- The contents of this material are confidential and 
#     proprietary to ClearOne Communications and may not be Reproduced, 
#     Published, or Disclosed to others without company authorization.
#
# Filename : getSysConfig.cgi
#
# Project : Timpanogos
#
# Description : script to return System Configuration Table to Web Page
#
# Owner : May Vilaiphanh
#
# Date : 11/19/2013
#
################################################################################  

use strict;
use warnings;

my $dir = "/disc"; #this directory contains system config file for all boxes in Timpanogos stack
my $str;
my $line;
my $headerTable;
my $bodyTable;
my $footerTable;
my @dirList;
my $glCount=0;
my $glTotals=0;


# don't know how may boxes in Timpanogos stack; thererfore, require actions
# to loop through box000-boxN directories to gather all info needed.              
if (opendir(DIR, $dir)) 
{
	@dirList 
  	= grep {
  	!/^\./                # not begins with a period 
 			      # or ( next if $file =~ m/^\./);
  	&& -d "$dir/$_"       # and is a file
 	 } readdir(DIR);
	close(DIR);
	
	if ( $#dirList == -1 )  # /disc is empty
	{
		print "Content-type:text/html\r\n\r\n"; #this string is sent to header part of web page 
		print "nodata"; # no mounted /data and "nodata" string is sent to the body part of web page	
	}
	else    #  /data not empty
	{
		# create Table Header
		$headerTable = "Content-type:text/html\r\n\r\n";	
		$headerTable .=  "<html>";
		$headerTable .=  "<body>" ;
		$headerTable .=  "<br><br>";
		$headerTable .= "<table border=1 cellpadding=10>";
		$headerTable .=  "<tr>";
		$headerTable .=  "<th>Box Type</th>";
		$headerTable .=  "<th>DID Switch</th>";
		$headerTable .=  "<th>Box Label</th>";
		$headerTable .=  "<th>Glink Order</th>";
		$headerTable .=  "<th>Glink Status</th>";
		$headerTable .=  "<th>IP Address</th>";
		$headerTable .=  "</tr>";
	
		$bodyTable = "";
	
		$footerTable .=  "</body>";
		$footerTable .=  "</html>";


		# Loop through all boxes in Timponogos stack and open all configuration files.
		# Extract fields and check error files on all boxes
		foreach my $eachDir (@dirList)
		{
			my $dirName = $dir. "\/" . $eachDir;
	
			if (opendir(DIR, $dirName))
			{
				while ( my $file = readdir(DIR)) 
				{
					# find specified file
					next unless ($file =~ m/config/);
	
					my $fileName = $dirName. "\/" . $file;
					# print "$filename\n";
					if (open(FD_IN, $fileName))
					{
		
						# Read the lines one by one
						while ( $line = <FD_IN> )
						{
							chomp($line);
							my ($boxType, $didNo, $boxLabel, $glOrder, $glError, $fpgaVersion, $glConn, $glTotal)=split(',',$line);
							
							$didNo = qx(cat $dirName/didsw);
							my $ipAddress = qx(ifconfig eth0 | grep "inet addr" | cut -d: -f2 | sed 's/Bcast//');
							
							# check all error files on each box
							my $didErr = $dirName. "\/" . "did_error";
							my $glErr = $dirName. "\/" . "gl_error";
							my $fwErr = $dirName. "\/" . "fw_error";
				
							my $showDid = ( -e $didErr ) ? "error" : "good";
							my $show_glError = ( -e $glErr ) ? "error" : "good";
							#my $show_fwError = ( -e $fwErr ) ? "error" : "good";
			
							# create Table Column/Rows
							$bodyTable .=  "<tr>";
							$bodyTable .=  "<td style=text-align:center;>$boxType</td>";
							if ( $showDid eq "error" )
							{
								$bodyTable .=  "<td style=text-align:center;color:red;>$didNo</td>";
							}
							else
							{
								$bodyTable .=  "<td style=text-align:center;color:black;>$didNo</td>";
							}
			
							$bodyTable .=  "<td style=text-align:center;>empty</td>";
							$bodyTable .=  "<td style=text-align:center;>$glOrder</td>";
		
							if ( $show_glError eq "error" )
							{
								$bodyTable .=  "<td style=text-align:center;color:red;>$show_glError</td>";
							}
							else
							{
								$bodyTable .=  "<td style=text-align:center;color:black;>$show_glError</td>";
							}
			
							$bodyTable .=  "<td style=text-align:center;color:black;>$ipAddress</td>";
							
							$glTotals = $glTotal;
							$glCount++;
							$bodyTable .=  "</tr>";		
			
						}
						close(FD_IN);
					}
				}
				closedir(DIR);
			}		
		}

		print "$headerTable";
		$bodyTable .= "<h2>System: total units = $glTotals started = $glCount  pending = " . ($glTotals - $glCount) . "</h2>";
		print "$bodyTable";
		print "$footerTable";		
			
	}
	
}






