<?php

abstract class JIBaseTable
{
    protected static $_instance = NULL;
    protected $_csv_file;
    protected $_csvcontent_arr = array();
    protected $_tableheader_arr = array();
    
    /*
     * Checks if the designated CSV file exists.
     * 
     * @return boolean
     */
    public function CSVFileExists()
    {
        return file_exists($this->_csv_file);
    }
    
    /*
     * Retrieves the header menu defined in the table.
     * 
     * @return array
     */
    public function GetTableHeaderMenus()
    {
        return $this->_tableheader_arr;
    }
    
    /*
     * Retrieve the table header menu and put it into an array.
     */
    protected function _GetTableHeader()
    {
        // Look for the line where the TABLE STARTS to determine
        // the position of the TABLE HEADER
        $needle = 'TABLE - START';
        $key = array_keys(array_filter($this->_csvcontent_arr, function($var) use ($needle){
            return strpos($var, $needle) !== false;
        }))[0] + 1; // Add 1 to get the correct table header position
        // Save table header menu inside the array
        $this->_tableheader_arr = explode(',', trim($this->_csvcontent_arr[$key]));
    }
    
    /*
     * Retrieves the single record row in the table.
     * 
     * @param string Searched String
     * 
     * @return string
     */
    protected function _GetTableRecordRow($needle='')
    {
        // Get the first record row found inside the table
        $tbl_record_row = array_values(array_filter($this->_csvcontent_arr, function($var) use ($needle){
            return strpos($var, $needle) !== false;
        }));
        return !empty($tbl_record_row) ? $tbl_record_row[0] : '';
    }
    
    /*
     * Retrieves the value of a column in a table record.
     * 
     * @param string Table Record row
     * @param string Table Column name
     * 
     * @return string Column value
     */
    protected function _GetTableRecordColumnValue($tbl_record_row='', $tbl_column_name='')
    {
        // Get the column position in the table header menu
        if(($column_index = array_search($tbl_column_name, $this->_tableheader_arr)) === FALSE) {
            return NULL; // If column name not found in the table header menu
        }
        // Split table record row by ","
        $tbl_record_row_arr = explode(',', $tbl_record_row);
        return trim($tbl_record_row_arr[$column_index]);
    }
}