<?php

class JIBoxLV
{
    private static $__instance = NULL;
    private $__csv_file = '/fwdata/live/stackLV.txt';
 //   private $__csv_file = 'boxLV.txt';
    private $__csvcontent_arr = array();
    private $__tableheader_arr = array();
    
    /*
     * Retrieve the object instance of the class
     * or create if nothing is set yet.
     */
    public static function GetInstance()
    {
        if(self::$__instance == NULL) {
            self::$__instance = new JIBoxLV();
        }
        return self::$__instance;
    }
    
    /*
     * Checks if the designated CSV file exists.
     * 
     * @return boolean
     */
    public function CSVFileExists()
    {
        return file_exists($this->__csv_file);
    }
    
    /*
     * Retrieves the header menu defined in the table.
     * 
     * @return array
     */
    public function GetTableHeaderMenus()
    {
        return $this->__tableheader_arr;
    }
    
    /*
     * Retrieve the table header menu and put it into an array.
     */
    private function __GetTableHeader()
    {
        // Look for the line where the TABLE STARTS to determine
        // the position of the TABLE HEADER
        $needle = 'TABLE - START';
        $key = array_keys(array_filter($this->__csvcontent_arr, function($var) use ($needle){
            return strpos($var, $needle) !== false;
        }))[0] + 1; // Add 1 to get the correct table header position
        // Save table header menu inside the array
        $this->__tableheader_arr = explode(',', trim($this->__csvcontent_arr[$key]));
    }
    
    /*
     * Retrieves the single record row in the table.
     * 
     * @param string Searched String
     * 
     * @return string
     */
    private function __GetTableRecordRow($needle='')
    {
        // Get the first record row found inside the table
        $tbl_record_row = array_values(array_filter($this->__csvcontent_arr, function($var) use ($needle){
            return strpos($var, $needle) !== false;
        }));
        return !empty($tbl_record_row) ? $tbl_record_row[0] : '';
    }
    
    /*
     * Retrieves the value of a column in a table record.
     * 
     * @param string Table Record row
     * @param string Table Column name
     * 
     * @return string Column value
     */
    private function __GetTableRecordColumnValue($tbl_record_row='', $tbl_column_name='')
    {
        // Get the column position in the table header menu
        if(($column_index = array_search($tbl_column_name, $this->__tableheader_arr)) === FALSE) {
            return NULL; // If column name not found in the table header menu
        }
        // Split table record row by ","
        $tbl_record_row_arr = explode(',', $tbl_record_row);
        return trim($tbl_record_row_arr[$column_index]);
    }
    
    /*
     * Authenticate username and password.
     * @param string username
     * @param string password
     * @return boolean (TRUE/FALSE)
     */
    public function ValidateCredentials($username='', $password='')
    {
        // Check for empty values
        if(empty($username) || empty($password)) {
            return FALSE;
        }
        // Get the position of BN, PN, PV in the table header menu
        $iBN = array_search('BN', $this->__tableheader_arr);
        $iPN = array_search('PN', $this->__tableheader_arr);
        $iPV = array_search('PV', $this->__tableheader_arr);
        // Set username info
        $unameinfo_arr = array();
        $unameinfo_arr[$iBN] = '9';
        $unameinfo_arr[$iPN] = '1';
        $unameinfo_arr[$iPV] = $username;
        // Set password info
        $pwordinfo_arr = array();
        $pwordinfo_arr[$iBN] = '9';
        $pwordinfo_arr[$iPN] = '2';
        $pwordinfo_arr[$iPV] = $password;
        // Convert username and password info into string
        $uname = implode(',', $unameinfo_arr);
        $pword = implode(',', $pwordinfo_arr);
        // Check if both info strings exists in the CSV table
        $unamerows = $this->__GetTableRecordRow($uname);
        $pwordrows = $this->__GetTableRecordRow($pword);
        // Username or Password was not found
        if(empty($unamerows) || empty($pwordrows)) {
            return FALSE;
        }
        // Validation successful
        return TRUE;
    }
    
    /*
     * Initialize object requirements.
     */
    private function __construct()
    {
        // If CSV file exists begin parsing
        if($this->CSVFileExists()) {
            // Get the CSV content
            $this->__csvcontent_arr =  file($this->__csv_file);
            // Initialize the table header menu
            $this->__GetTableHeader();
        }
    }
}
