<?php

class JITelnetClient
{
    private static $__instance = NULL;
    private $__socket = NULL;
    public $__buffer = '';
    private $__errno;
    private $__errstr;
    private $__host;
    private $__port;
    private $__timeout;
    // Special characters
    private $__NULL;
    private $__CR;
    private $__DC1;
    private $__IAC;
    // Constant variables
    const TELNET_ERROR = FALSE;
    const TELNET_OK = TRUE;
    
    /*
     * Retrieve the object instance of the class
     * or create if nothing is set yet.
     * 
     * @access public
     * @param   string  $host Host name or IP addres
     * @param   int     $port TCP port number
     * @param   int     $timeout Connection timeout in seconds
     * @return class instance
     */
    public static function GetInstance($host = '127.0.0.1', $port = 23, $timeout = 10)
    {
        if(self::$__instance == NULL) {
            self::$__instance = new JITelnetClient($host, $port, $timeout);
        }
        return self::$__instance;
    }
    
    /*
     * Attempts connection to remote host. Returns TRUE if sucessful.
     * 
     * @access    public
     * @return    boolean
     */
    public function Connect()
    {
//        // If hostname is specified instead of an IP address
//        if(!preg_match('/([0-9]{1,3}\\.){3,3}[0-9]{1,3}/', $this->__host)) {
//            // Get the IP address of the hostname
//            $ip = gethostbyname($this->__host);
//            // No IP address being retrieved
//            if($this->__host == $ip) {
//                throw new Exception('Cannot resolve '.$this->__host.'.');
//            }
//            else {
//                $this->host = $ip;
//            }
//        }
//        echo 'Opening new socket for telnet connection<br/>';
//        // Create and open new socket
//        $this->__socket = fsockopen($this->__host, $this->__port, $this->__errno, $this->__errstr, $this->__timeout);
//        
//        if(!$this->__socket) {
//            throw new Exception('Cannot connect to '.$this->__host.' on port '.$this->__port.'.<br/>'.$this->__errstr.'('.$this->__errno.')');
//        }
//        // Get login response
//        $this->__Read('Username: ');
        
        /* Create a TCP/IP socket */
        $this->__socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP);
        
        if($this->__socket === false) {
            echo 'socket_create() failed: reason: '.socket_strerror(socket_last_error()).PHP_EOL;
            return self::TELNET_ERROR;
        }
        
        $result = socket_connect($this->__socket, $this->__host, $this->__port);
        
        if($result === false) {
            echo 'socket_connect() failed.\nReason: ('.$result.') '.socket_strerror(socket_last_error($this->__socket)).PHP_EOL;
            return self::TELNET_ERROR;
        }
        
        return $this->__Read('Username:') ? self::TELNET_OK : self::TELNET_ERROR;
    }
    
    /* 
     * Attempts login to remote host.
     * 
     * @access  public
     * @param   string  $username Username
     * @param   string  $password Password
     * @return  bool
     */
    public function Login($username = FALSE, $password = FALSE)  
    {
        try {
            if($username) {
                $this->__Write($username);
                if($this->__Read('Password:')) {
                    if($password) {
                        $this->__Write($password);
                        if($this->__Read('=>', FALSE)) {
                            return self::TELNET_OK;
                        }
                    }
                }
            }
        }
        catch(Exception $e) {
            throw new Exception('Login failed.');
        }
        
        return self::TELNET_ERROR;
    }
    
    /* 
     * Sends serial command to the host.
     * 
     * @access  public
     * @param   string  $buffer Command string
     * @return  bool
     */
    public function SendCommand($buffer = '')  
    {
        try {
            // Remove white spaces at the beginning and end
            $buffer = trim($buffer);
            
            if($buffer) {
                $this->__Write($buffer);
                // Read the response first
                $status = $this->__Read('');
                $status = $this->__Read('', FALSE);
                // Split each commands
                $cmd_arr = explode(' ', $buffer);
                if(strpos($this->__buffer, strtoupper($cmd_arr[0])) !== FALSE && strpos($this->__buffer, '=>') !== FALSE) {
                    return self::TELNET_OK;
//                    return self::TELNET_ERROR;
                }
            }
        }
        catch(Exception $e) {
            throw new Exception('Sending command failed.');
        }
        
        return self::TELNET_ERROR;
    }
    
    /*
     * Get the latest server response
     * 
     * @access public
     * @return string
     */
    public function GetLastServerResponse()
    {
        return nl2br($this->__buffer);
    }
    
    /*
     * Close the socket used in connecting to the telnet server
     * 
     * @access public
     * @return void
     */
    public function Close()
    {
        if($this->__socket) {
            socket_close($this->__close);
        }
        $this->__socket = NULL;
    }
    
    /*
     * Constructor. Initializes host, port and timeout parameters
     * defaults to localhost port 23 (standard telnet port)
     * 
     * @access  private
     * @param   string  $host Host name or IP addres
     * @param   int     $port TCP port number
     * @param   int     $timeout Connection timeout in seconds
     * @return  void
     */
    private function __construct($host = '127.0.0.1', $port = 23, $timeout = 10)
    {
        $this->__host = $host;
        $this->__port = $port;
        $this->__timeout = $timeout;
        // Set special characters
        $this->__NULL = chr(0);
        $this->__CR = chr(13);
        $this->__DC1 = chr(17);
        $this->__IAC = chr(255);
    }
    
    /*
     * Sends packets.
     * 
     * @access  private
     * @param   string  $buffer Stuff to write to socket
     * @param   bool    $addNewLine Default true, adds newline to the command
     * @param   bool    $clearBuffer Default true, clear buffer from last command
     * @return  bool
     */
    private function __Write($buffer = '', $addNewLine = TRUE, $clearBuffer = TRUE) 
    {
        if(!$this->__socket) {
            throw new Exception('Telnet connection closed.');
        }
        
        if($clearBuffer) {
            $this->__ClearBuffer();
        }
        
        if($addNewLine == TRUE) {
            $buffer .= $this->__CR;
        }
        
        if(socket_write($this->__socket, $buffer, strlen($buffer)) === FALSE) {
            throw new Exception('Error writing to socket.');
        }
//        if(!fwrite($this->__socket, $buffer) < 0) {
//            throw new Exception('Error writing to socket.');
//        }
        
        return self::TELNET_OK;
    }
    
    /* 
     * Reads characters from the socket and adds them to command buffer.
     * Handles telnet control characters. Stops when prompt is ecountered.
     * 
     * @access  private
     * @param   string  $prompt
     * @param   bool    $clearBuffer
     * @return  bool
     */
    private function __Read($prompt='', $clearBuffer=true) 
    {
        if(!$this->__socket) {
            throw new Exception('Server connection closed.');
        }
        
        if($clearBuffer)
            $this->__ClearBuffer();
        
        $this->__buffer .= socket_read($this->__socket, 2048);
        
        return strpos($this->__buffer, $prompt) !== FALSE ? self::TELNET_OK : self::TELNET_ERROR;
//        do {
//            $c = $this->__GetC();
//            
//            if($c === FALSE) {
//                throw new Exception('Can\'t find the requested prompt: '.$prompt);
//            }
//            
//            if($c == $this->__IAC) {
//                if($this->negotiateTelnetOptions()) {
//                    continue; 
//                }
//            }
//            
//            $this->__buffer .= $c;
//            
//            if(substr($this->__buffer, -10) == " --More-- ") {
//                $this->write(NULL, TRUE, FALSE);
//            }
//            elseif(@preg_match($prompt, $this->__buffer)) {
//            if($c === FALSE && strpos($this->__buffer, $prompt) !== FALSE) {
////                echo $this->__buffer.'<br/>';
//                return self::TELNET_OK;
//            }
//        }
//        while($c != $this->__NULL || $c != $this->__DC1);
    }
    
    /*
     * Clears internal command buffer
     * 
     * @access  private
     * @return
     */
    private function __ClearBuffer()
    {
        $this->__buffer = '';
    }
    
    /*
     * Gets character from the socket
     * 
     * @access  private
     * @return  character
     */
    private function __GetC()
    {
        @socket_set_timeout($this->__socket, $this->__timeout);
        
        return fgetc($this->__socket);
    }
    
    private function __PrintError($msg='')
    {
        $error_code = socket_last_error();
        $error_msg = socket_strerror($error_code);
        die($msg.': ['.$error_code.'] '.$error_msg.'<br/>');
    }
}